# Overview: renderJS for jupyter notebook via extension

This repo contains two seperate things:
1. A [jupyter frontend extension](http://jupyter-notebook.readthedocs.io/en/latest/extending/frontend_extensions.html) 
also called nbextension containing the javascript and html required to load arbitrary gadgets. See folder 
`renderjs_nbextension/`.
2. An [ipython extension](https://ipython.org/ipython-doc/3/config/extensions/index.html) providing necessary python the commands to initiate the renderjs and controll the loaded gadgets. This happens directly from the notebook (cells). See folder `renderjs_ipyextension/`.

To make this work both the nbextension and the ipython-extension need to be installed and enabled.


# Install
TODO

# Use
All of the following python calls are made from the notebook itself.
## Loading the ipython extension:
```python
%load_ext renderjs_extension
```

## Import and initialize the extension:
```python
import renderjs_extension as rjs
rjs.init_renderjs()
```

## Load a renderjs gadget:
```python
myGadget = rjs.load_gadget("https://my.gadget.url/gadget.html")
```

## Destroy a renderjs gadget:
```python
myGadget.destroy()
```
If he cell defining `myGadget` is executed again, the current `myGadget` is automatically destroyed and a new one created.

## Call a `declared_method` of a loaded gadget:
```python
arg1 = "Hello"
arg2 = 'World'
arg3 = 2016
arg4 = { "yes": "no!" }
myGadget.call_declared_method("method_name", arg1, arg2, arg3, arg4)
```
Arguments can be arbitrary as long as it is python-compatible. They get converted to js-arguments via pythons JSON command.

# Development
The extension is split into a ipython extension and a nbextension für jupyter. This is necessary because both JS in the frontend
and python in the backend are required to load a renderjs gadget.

## ipython extension (backend)
Here all the functionality for interacting with the extension via jupyter notebook cells is exposed. Unfortunately, ipython
has no knowledge of the kernel and therefore no knowledge of the jupyter frontend.  
Still we want to execute javascipt in the frontend. To solve this issue, ipython is used to inject inline
javascipt into the page via ipython display. This is a rather dirty hack but I could not find a better solution as of now.
The inject JS code get's executed and fires an event to which the frontend-extension listens to.

Gadgets are represented by a python-object and carry a uid to identify them.

## nbextension (frontend)
Nothing happens on-load of the frontend part. Everything is controlled by the backend (mostly via events fired from the ipython). 
Exception to event-based communication is the initial and controller gadget (loading_gadget) 
which is injected into the page directly and which is initialized by a manual bootstrap of renderjs (since the page is
already loaded; normal bootstrapping happens on-load). 
The loading_gadget is responsible for loading other 
(actual) gadgets and pass information from the backend to them.  
The loading_gadget declares a renderjs service for each possible event which could be fired from the backend
(such as call_declared_method, destroy_gadget, ...).

Multiple gadgets are possible which are all controlled by the same loading_gadget. To differentiate between gadgets
each gadget sends along a uid (generated in the backend and member-variable of the python gadget-object)
and the loading_gadget determines the right gadget (by .getDeclaredGadget(uid)) and passes along the function call.